
//////////////////////////////////////////////////
//  Oracle Data Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  ODAC Design
//////////////////////////////////////////////////

{$IFNDEF CLR}
{$I Odac.inc}

unit OraDesign;
{$ENDIF}
interface
uses
  SysUtils, Classes,
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
{$IFDEF CLR}
  Graphics, Controls, Forms, Dialogs, StdCtrls, TypInfo,
  Borland.Vcl.Design.DesignEditors, Borland.Vcl.Design.DesignIntf,
  Borland.Vcl.Design.FldLinks,
  {$IFDEF BUILD_DCMEMO}OraMemo,{$ENDIF} OdacMenu,
{$ELSE}
  Graphics, Controls, Forms, Dialogs, StdCtrls, TypInfo,
{$IFDEF FPC}
  PropEdits, ComponentEditors,
{$ELSE}
  {$IFDEF VER6P}DesignIntf, DesignEditors,{$ELSE}DsgnIntf,{$ENDIF}
  {$IFNDEF BCB}{$IFDEF VER5P}FldLinks,{$ENDIF} ColnEdit,{$ENDIF}
  {$IFDEF BUILD_DCMEMO}OraMemo,{$ENDIF} OdacMenu,
{$ENDIF}
{$ENDIF}
{$IFDEF DBTOOLS}
  DBToolsClient,
{$ENDIF}
  CRFunctions, CRTypes, DBAccess, CRDesign, DADesign, Ora,
{$IFNDEF STD}
  OraPackage, OraAQ, OraTransaction,
{$ENDIF}
  OdacVcl, OraSmart;
  
type
{ ------------  ODAC property editors ----------- }

  TSmartQueryKeyFieldsEditor = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function AutoFill: boolean; override;
  end;

  TOraDataSetKeySequenceEditor = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function AutoFill: boolean; override;
  end;

  TOraTableKeyFieldsEditor = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function AutoFill: boolean; override;
  end;

{$IFDEF WIN32}
  TOraSessionHomeNameEditor = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;
{$ENDIF}

  TOraSessionOptionsCharsetEditor = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function AutoFill: boolean; override;
  end;

{$IFNDEF STD}
  TOraPackageNameEditor = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function AutoFill: boolean; override;
  end;

  TOraQueueNameEditor = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function AutoFill: boolean; override;
  end;

  TOraQueueTableNameEditor = class (TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
    function AutoFill: boolean; override;
  end;

  TOraQueueAdminQueueTableNameEditor = class (TOraQueueTableNameEditor)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TQueueMessagePropertiesDelayEditor = class (TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TQueueMessagePropertiesExpirationEditor = class (TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TDequeueOptionsWaitTimeoutEditor = class (TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TOraQueueAdminRetentionTimeEditor = class (TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TOraTraceMaxTraceFileSizeEditor = class (TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;
{$ENDIF}  

  TOraConnectDialogPropertyEditor = class(TComponentProperty)
  private
    FCheckProc: TGetStrProc;
    procedure CheckComponent(const Value: string);
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ ------------  ODAC component editors ----------- }

  TOraSessionEditor = class (TDAConnectionEditor)
  protected
    procedure InitVerbs; override;
  end;

  TOraDataSetEditor = class (TDAComponentEditor)
  end;

  TOraQueryEditor = class (TOraDataSetEditor)
  protected
  {$IFNDEF FPC}
    procedure ConvertToSmartQuery;
  {$ENDIF}
    procedure InitVerbs; override;
  end;

  TSmartQueryEditor = class (TOraDataSetEditor)
  protected
  {$IFNDEF FPC}
    procedure ConvertToOraQuery;
  {$ENDIF}
    procedure InitVerbs; override;
  end;

  TOraTableEditor = class (TOraDataSetEditor)
  protected
    procedure InitVerbs; override;
  end;

  TOraStoredProcEditor = class (TOraDataSetEditor)
  protected
  {$IFNDEF FPC}
    procedure ConvertToOraQuery;
  {$ENDIF}
    procedure InitVerbs; override;
  end;

  TOraSQLEditor = class (TDASQLEditor)
  protected
    procedure InitVerbs; override;
  end;

  TOraScriptEditor = class (TDAScriptEditor)
  protected
    procedure InitVerbs; override;
  end;

{$IFNDEF STD}  
  TOraErrorHandlerEditor = class (TDAComponentEditor)
  protected
    procedure InitVerbs; override;
  end;
{$ENDIF}

  TOraUpdateSQLEditor = class (TDAUpdateSQLEditor)
  protected
    procedure InitVerbs; override;
  end;

{$IFNDEF STD}
  TOraQueueTableEditor = class (TDAComponentEditor)
  private
    procedure ReadQueueTableProperties;
    procedure CreateQueueTable;
  protected
    procedure InitVerbs; override;
  end;

  TOraQueueAdminEditor = class (TDAComponentEditor)
  private
    procedure ReadQueueProperties;
    procedure CreateQueue;
  protected
    procedure InitVerbs; override;
  end;

{$IFDEF DBTOOLS}
  TOraPackageEditor = class (TDAComponentEditor)
  protected
    procedure InitVerbs; override;
  end;
{$ENDIF}
{$ENDIF}

{$IFNDEF FPC}
  TOraConnectionList = class (TDAConnectionList)
  protected
    function GetConnectionType: TCustomDAConnectionClass; override;
  end;

{$IFDEF VER6P}
  TOraDesignNotification = class(TDADesignNotification)
  public
    procedure ItemInserted(const ADesigner: IDesigner; AItem: TPersistent); override;
    function CreateConnectionList: TDAConnectionList; override;
    function GetConnectionPropertyName: string; override;
  end;
{$ENDIF}
{$ENDIF}

procedure Register;

implementation

uses
  Db, OraSQLMonitor,
  OraQueryEditor, SmartQueryEditor, OraTableEditor, OraStoredProcEditor,
  OraSQLEditor, OraSessionEditor, OraUpdateSQLEditor,
  OraCall, OraClasses, {$IFNDEF FPC}MemDS{$ELSE}MemDataSet{$ENDIF}, VirtualTable, OraError,
{$IFDEF MSWINDOWS}
   DBMonitorClient, ShellApi,
{$ENDIF}
{$IFDEF WIN32}
{$IFNDEF FPC}
  ToolIntf, ExptIntf, EditIntf,
{$ENDIF}
{$ENDIF}
{$IFDEF CLR}
  Borland.VCL.Design.DSDesign, Borland.Vcl.Design.ColnEdit, WinUtils,
{$ELSE}
{$IFNDEF BCB}
{$IFNDEF FPC}
  DSDesign,
{$ENDIF}
{$ENDIF}
{$ENDIF}
{$IFDEF VER6P}
  Variants,
{$ENDIF}
  OraScript, DAScriptEditor,
{$IFNDEF STD}
{$IFDEF MSWINDOWS}
{$IFNDEF FPC}
  OraAlerter,
{$ENDIF}
{$ENDIF}
  DALoader, OraLoader, OraErrHand, OraErrorHandlerEditor,
{$ENDIF}
  CRAccess, OraDesignUtils, DAConsts;

{ TSmartQueryKeyFieldsEditor }

function TSmartQueryKeyFieldsEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TSmartQueryKeyFieldsEditor.AutoFill: boolean;
begin
  Result := False;
end;

procedure TSmartQueryKeyFieldsEditor.GetValues(Proc: TGetStrProc);
var
  Query: TSmartQuery;
  List: TStrings;
  i:integer;
  TableName: string;
  TablesInfo: TCRTablesInfo;
  SQLInfo: TSQLInfo;
begin
  Query := TSmartQuery(GetComponent(0));
  TableName := Query.UpdatingTable;
  if TableName = '' then begin
    TablesInfo := TCRTablesInfo.Create(TCRTableInfo); //TODO
    try
      TDBAccessUtils.CheckIRecordSet(Query);
      SQLInfo := TDBAccessUtils.GetSQLInfo(Query);
      SQLInfo.ParseTablesInfo(Query.FinalSQL, TablesInfo);
      if TablesInfo.Count > 0 then
        TableName := TablesInfo[0].TableName;
    finally
      TablesInfo.Free;
    end;
  end;

  if TableName <> '' then begin
    List := TStringList.Create;
    try
      TSmartQuery(GetComponent(0)).GetKeyList(TableName, List);
      for i := 0 to List.Count - 1 do
        Proc(List[i]);
    finally
      List.Free;
    end;
  end;
end;

{ TOraDataSetKeySequenceEditor }

function TOraDataSetKeySequenceEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TOraDataSetKeySequenceEditor.AutoFill: boolean;
begin
  Result := False;
end;

procedure TOraDataSetKeySequenceEditor.GetValues(Proc: TGetStrProc);
var
  List: TStrings;
  i: integer;
  Session: TOraSession;
begin
  List := TStringList.Create;
  try
    Session := TOraDataSet(GetComponent(0)).Session;
    if Session = nil then
      Session := DefaultSession;
    if Session = nil then
      exit;
    Session.GetSequenceNames(List);
    for i := 0 to List.Count - 1 do
      Proc(List[i]);
  finally
    List.Free;
  end;
end;

{ TOraTableKeyFieldsEditor }

function TOraTableKeyFieldsEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TOraTableKeyFieldsEditor.AutoFill: boolean;
begin
  Result := False;
end;

procedure TOraTableKeyFieldsEditor.GetValues(Proc: TGetStrProc);
var
  List: TStrings;
  i: integer;
begin
  List := TStringList.Create;
  try
    TOraTable(GetComponent(0)).GetKeyList(TOraTable(GetComponent(0)).TableName, List);
    for i := 0 to List.Count - 1 do
      Proc(List[i]);
    Proc('ROWID');
  finally
    List.Free;
  end;
end;

{$IFDEF WIN32}
{ TOraSessionHomeNameEditor }

function TOraSessionHomeNameEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TOraSessionHomeNameEditor.GetValues(Proc: TGetStrProc);
var
  i: integer;
begin
/// fill home list with info
  for i := 0 to OracleHomes.Count - 1 do
    Proc(OracleHomes[i].Name);
end;
{$ENDIF}

{ TOraSessionOptionsCharsetEditor }

function TOraSessionOptionsCharsetEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TOraSessionOptionsCharsetEditor.AutoFill: boolean;
begin
  Result := False;
end;

procedure TOraSessionOptionsCharsetEditor.GetValues(Proc: TGetStrProc);
var
  List: TStrings;
  i: integer;
begin
  List := TStringList.Create;
  try
    TOraUtils.GetObjectList(TOraUtils.FOwner(TOraSessionOptions(GetComponent(0))), List,
      'SELECT Value FROM V$NLS_VALID_VALUES WHERE Parameter = ''CHARACTERSET'' ORDER BY Value');
    for i := 0 to List.Count - 1 do
      Proc(List[i]);
  finally
    List.Free;
  end;
end;

{ TOraPackageNameEditor }
{$IFNDEF STD}
function TOraPackageNameEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TOraPackageNameEditor.AutoFill: boolean;
begin
  Result := False;
end;

procedure TOraPackageNameEditor.GetValues(Proc: TGetStrProc);
var
  List: TStrings;
  i: integer;
  Session: TOraSession;
begin
  List := TStringList.Create;
  try
    Session := TOraPackage(GetComponent(0)).Session;
    if Session <> nil then
      TOraPackage(GetComponent(0)).Session.GetPackageNames(List, True)
    else
      DatabaseError(SConnectionNotDefined);
    for i := 0 to List.Count - 1 do
      Proc(List[i]);
  finally
    List.Free;
  end;
end;

{ TOraQueueNameEditor }

function TOraQueueNameEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TOraQueueNameEditor.AutoFill: boolean;
begin
  Result := False;
end;

procedure TOraQueueNameEditor.GetValues(Proc: TGetStrProc);
var
  Session: TOraSession;
  List: TStrings;
  i: integer;
begin
  if GetComponent(0) is TOraQueue then
    Session := TOraQueue(GetComponent(0)).Session
  else
    Session := TOraQueueAdmin(GetComponent(0)).Session;
  List := TStringList.Create;
  try
    TOraUtils.GetObjectList(Session, List,
      'SELECT owner || ''.'' || name FROM sys.all_queues ' +
      'ORDER BY owner, name');
    for i := 0 to List.Count - 1 do
      Proc(List[i]);
  finally
    List.Free;
  end;
end;

{ TOraQueueTableNameEditor }

function TOraQueueTableNameEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TOraQueueTableNameEditor.AutoFill: boolean;
begin
  Result := False;
end;

procedure TOraQueueTableNameEditor.GetValues(Proc: TGetStrProc);
var
  Session: TOraSession;
  List: TStrings;
  i: integer;
begin
  if GetComponent(0) is TOraQueueTable then
    Session := TOraQueueTable(GetComponent(0)).Session
  else
    Session := TOraQueueAdmin(GetComponent(0)).Session;
  if Session = nil then
    DatabaseError(SConnectionNotDefined);
  List := TStringList.Create;
  try
    TOraUtils.GetObjectList(Session, List,
      'SELECT owner || ''.'' || queue_table FROM sys.all_queue_tables ' +
      'ORDER BY owner, queue_table');
    for i := 0 to List.Count - 1 do
      Proc(List[i]);
  finally
    List.Free;
  end;
end;

{ TOraQueueAdminQueueTableNameEditor }

procedure TOraQueueAdminQueueTableNameEditor.GetValues(Proc: TGetStrProc);
var
  i: integer;
begin
  for i := 0 to QueueTableList.Count - 1 do
    Proc(QueueTableList[i].QueueTableName);

  inherited;
end;

{ TQueueMessagePropertiesDelayEditor }

function TQueueMessagePropertiesDelayEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TQueueMessagePropertiesDelayEditor.GetValue: string;
begin
  case GetOrdValue of
    AQ_NO_DELAY:
      Result := 'AQ_NO_DELAY';
  else
    Result := inherited GetValue;
  end;
end;

procedure TQueueMessagePropertiesDelayEditor.SetValue(const Value: string);
begin
  if AnsiUpperCase(Trim(Value)) = 'AQ_NO_DELAY' then
    SetOrdValue(AQ_NO_DELAY)
  else
    inherited;
end;

procedure TQueueMessagePropertiesDelayEditor.GetValues(Proc: TGetStrProc);
begin
  Proc('AQ_NO_DELAY');
end;

{ TQueueMessagePropertiesExpirationEditor }

function TQueueMessagePropertiesExpirationEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TQueueMessagePropertiesExpirationEditor.GetValue: string;
begin
  case GetOrdValue of
    AQ_NEVER :
      Result := 'AQ_NEVER';
  else
    Result := inherited GetValue;
  end;
end;

procedure TQueueMessagePropertiesExpirationEditor.SetValue(const Value: string);
begin
  if AnsiUpperCase(Trim(Value)) = 'AQ_NEVER' then
    SetOrdValue(AQ_NEVER)
  else
    inherited;
end;

procedure TQueueMessagePropertiesExpirationEditor.GetValues(Proc: TGetStrProc);
begin
  Proc('AQ_NEVER');
end;

{ TDequeueOptionsWaitTimeoutEditor }

function TDequeueOptionsWaitTimeoutEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TDequeueOptionsWaitTimeoutEditor.GetValue: string;
begin
  case GetOrdValue of
    AQ_FOREVER:
      Result := 'AQ_FOREVER';
    AQ_NO_WAIT:
      Result := 'AQ_NO_WAIT';
  else
    Result := inherited GetValue;
  end;
end;

procedure TDequeueOptionsWaitTimeoutEditor.SetValue(const Value: string);
begin
  if AnsiUpperCase(Trim(Value)) = 'AQ_FOREVER' then
    SetOrdValue(AQ_FOREVER)
  else
    if AnsiUpperCase(Trim(Value)) = 'AQ_NO_WAIT' then
      SetOrdValue(AQ_NO_WAIT)
    else
      inherited;
end;

procedure TDequeueOptionsWaitTimeoutEditor.GetValues(Proc: TGetStrProc);
begin
  Proc('AQ_FOREVER');
  Proc('AQ_NO_WAIT');
end;

{ TOraQueueAdminRetentionTimeEditor }

function TOraQueueAdminRetentionTimeEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TOraQueueAdminRetentionTimeEditor.GetValue: string;
begin
  case GetOrdValue of
    AQ_INFINITE:
      Result := 'AQ_INFINITE';
  else
    Result := inherited GetValue;
  end;
end;

procedure TOraQueueAdminRetentionTimeEditor.SetValue(const Value: string);
begin
  if AnsiUpperCase(Trim(Value)) = 'AQ_INFINITE' then
    SetOrdValue(AQ_INFINITE)
  else
    inherited;
end;

procedure TOraQueueAdminRetentionTimeEditor.GetValues(Proc: TGetStrProc);
begin
  Proc('AQ_INFINITE');
end;

{ TOraTraceMaxTraceFileSizeEditor }

function TOraTraceMaxTraceFileSizeEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList];
end;

function TOraTraceMaxTraceFileSizeEditor.GetValue: string;
begin
  case GetOrdValue of
    DEFAULT_TRACE_FILE_SIZE:
      Result := 'DEFAULT';
    UNLIMITED_TRACE_FILE_SIZE:
      Result := 'UNLIMITED';
  else
    Result := inherited GetValue;
  end;
end;

procedure TOraTraceMaxTraceFileSizeEditor.SetValue(const Value: string);
begin
  if AnsiUpperCase(Trim(Value)) = 'DEFAULT' then
    SetOrdValue(DEFAULT_TRACE_FILE_SIZE)
  else
    if AnsiUpperCase(Trim(Value)) = 'UNLIMITED' then
      SetOrdValue(UNLIMITED_TRACE_FILE_SIZE)
    else
      inherited;
end;

procedure TOraTraceMaxTraceFileSizeEditor.GetValues(Proc: TGetStrProc);
begin
  Proc('DEFAULT');
  Proc('UNLIMITED');
end;
{$ENDIF}

{ TOraConnectDialogPropertyEditor }

procedure TOraConnectDialogPropertyEditor.CheckComponent(const Value: string);
var
  Component: TComponent;
begin
  Component := {$IFDEF FPC}PropertyHook{$ELSE}Designer{$ENDIF}.GetComponent(Value);
  if Component <> nil then begin
    if not ({$IFDEF VER17P}(GetIsClassByName(Component, 'TConnectDialogFmx')) or {$ENDIF}(Component is TConnectDialog)) then
      Exit;
  end;
  FCheckProc(Value);
end;

procedure TOraConnectDialogPropertyEditor.GetValues(Proc: TGetStrProc);
begin
  FCheckProc := Proc;
  inherited GetValues(CheckComponent);
end;

{ TOraSessionEditor }

procedure TOraSessionEditor.InitVerbs;
begin
  inherited;
  AddVerb('Session Editor...', TOraSessionEditorForm, TOraDesignUtils);
{$IFDEF DBTOOLS}
  AddDBToolsVerbs([dbtFindInDatabaseExplorer]);
{$ENDIF}
end;

{$IFNDEF FPC}
function TOraConnectionList.GetConnectionType: TCustomDAConnectionClass;
begin
  Result := TOraSession;
end;

{$IFDEF VER6P}
procedure TOraDesignNotification.ItemInserted(const ADesigner: IDesigner; AItem: TPersistent);
begin
  if (AItem <> nil) and ((AItem is TOraDataSet)
    or (AItem is TOraScript)
{$IFNDEF STD}
{$IFDEF MSWINDOWS}
    or (AItem is TOraAlerter)
{$ENDIF}
    or (AItem is TOraLoader) or (AItem is TOraErrorHandler)
    or (AItem is TOraDataSource)
    or (AItem is TCustomOraPackage)
    or (AItem is TOraTransaction)
    or (AItem is TOraQueue) or (AItem is TOraQueueTable) or (AItem is TOraQueueAdmin)
    or (AItem is TOraTrace)
{$ENDIF}
    or (AItem is TOraSQL)
    or (AItem is TOraMetaData))
  then
    FItem := AItem;
end;

function TOraDesignNotification.CreateConnectionList: TDAConnectionList;
begin
  Result := TOraConnectionList.Create;
end;

function TOraDesignNotification.GetConnectionPropertyName: string;
begin
{$IFNDEF STD}
  if FItem is TOraTransaction then
    Result := 'DefaultSession'
  else
{$ENDIF}  
    Result := 'Session'
end;
{$ENDIF}
{$ENDIF}

{ TOraQueryEditor }

{$IFNDEF FPC}
procedure TOraQueryEditor.ConvertToSmartQuery;
begin
  if Designer <> nil then
    ConvertToClass(Self.Designer, Component, TSmartQuery);
end;
{$ENDIF}

procedure TOraQueryEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('OraQuery E&ditor...', TOraQueryEditorForm, TOraDesignUtils);
  AddVerb('Data Editor...', ShowDataEditor);
{$IFNDEF FPC}
  AddVerb('Convert to TSmartQuery', ConvertToSmartQuery);
{$ENDIF}
{$IFDEF DBTOOLS}
  AddDBToolsVerbs([dbtEditSelectSql, dbtEditInsertSql, dbtEditUpdateSql, dbtEditDeleteSql, dbtEditLockSql, dbtEditRefreshSql, dbtQueryBuilder, dbtDebugSql, dbtRetrieveData, dbtEditRecCountSQL]);
{$ENDIF}
  inherited;
end;

{ TSmartQueryEditor }

{$IFNDEF FPC}
procedure TSmartQueryEditor.ConvertToOraQuery;
begin
  if Designer <> nil then
    ConvertToClass(Self.Designer, Component, TOraQuery);
end;
{$ENDIF}

procedure TSmartQueryEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('SmartQuery E&ditor...', TSmartQueryEditorForm, TOraDesignUtils);
  AddVerb('Data Editor...', ShowDataEditor);
{$IFNDEF FPC}
  AddVerb('Convert to TOraQuery', ConvertToOraQuery);
{$ENDIF}
{$IFDEF DBTOOLS}
  AddDBToolsVerbs([dbtEditSelectSql, dbtEditInsertSql, dbtEditUpdateSql, dbtEditDeleteSql, dbtEditLockSql, dbtEditRefreshSql, dbtQueryBuilder, dbtDebugSql, dbtRetrieveData, dbtEditRecCountSQL]);
{$ENDIF}
  inherited;
end;

{ TOraTableEditor }

procedure TOraTableEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('OraTable E&ditor...', TOraTableEditorForm, TOraDesignUtils);
  AddVerb('Data Editor...', ShowDataEditor);
{$IFDEF DBTOOLS}
  AddDBToolsVerbs([dbtEditDatabaseObject, dbtFindInDatabaseExplorer]);
{$ENDIF}

  inherited;
end;

{ TOraStoredProcEditor }

{$IFNDEF FPC}
procedure TOraStoredProcEditor.ConvertToOraQuery;
begin
  if Designer <> nil then
    ConvertToClass(Self.Designer, Component, TOraQuery);
end;
{$ENDIF}

procedure TOraStoredProcEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('StoredProc E&ditor...', TOraStoredProcEditorForm, TOraDesignUtils);
{$IFNDEF FPC}
  AddVerb('Convert to TOraQuery', ConvertToOraQuery);
{$ENDIF}
{$IFDEF DBTOOLS}
  AddDBToolsVerbs([dbtFindInDatabaseExplorer, dbtEditDatabaseObject, dbtEditInsertSql, dbtEditUpdateSql, dbtEditDeleteSql, dbtEditLockSql, dbtEditRefreshSql, dbtDebugSql, dbtCompile, dbtCompileDebug, dbtEditRecCountSQL]);
{$ENDIF}

  inherited;
end;

{ TOraSQLEditor }

procedure TOraSQLEditor.InitVerbs;
begin
  inherited;
  AddVerb('OraSQL E&ditor...', TOraSQLEditorForm, TOraDesignUtils);
{$IFDEF DBTOOLS}     
  AddDBToolsVerbs([dbtEditSql, dbtDebugSql]);
{$ENDIF}
end;

{ TOraScriptEditor }

procedure TOraScriptEditor.InitVerbs;
begin
  inherited;
  AddVerb('OraScript E&ditor...', TDAScriptEditorForm, TOraDesignUtils);
{$IFDEF DBTOOLS}
  AddDBToolsVerbs([dbtEditSql, dbtDebugSql]);
{$ENDIF}
end;

{ TOraErrorHandlerEditor }
{$IFNDEF STD}
procedure TOraErrorHandlerEditor.InitVerbs;
begin
  AddVerb('OraErrorHandler E&ditor...', TOraErrorHandlerEditorForm, TOraDesignUtils);
end;
{$ENDIF}

{ TOraUpdateSQLEditor }

procedure TOraUpdateSQLEditor.InitVerbs;
begin
  AddVerb('OraUpdateSQL E&ditor...', TOraUpdateSQLEditorForm, TOraDesignUtils);
{$IFDEF DBTOOLS}
  AddDBToolsVerbs([dbtEditInsertSql, dbtEditUpdateSql, dbtEditDeleteSql, dbtEditLockSql, dbtEditRefreshSql]);
{$ENDIF}

  inherited;
end;

{ TOraQueueTableEditor }
{$IFNDEF STD}
procedure TOraQueueTableEditor.InitVerbs;
begin
  AddVerb('Read Queue Table properties', ReadQueueTableProperties);
  AddVerb('Create Queue Table', CreateQueueTable);
  inherited;
end;

procedure TOraQueueTableEditor.ReadQueueTableProperties;
begin
  TOraQueueTable(Component).ReadQueueTableProperties;
  if Self.Designer <> nil then
    Self.Designer.Modified;
end;

procedure TOraQueueTableEditor.CreateQueueTable;
begin
  TOraQueueTable(Component).CreateQueueTable;
  ShowMessage('Queue Table created');
end;

{ TOraQueueAdminEditor }

procedure TOraQueueAdminEditor.InitVerbs;
begin
  AddVerb('Read Queue properties', ReadQueueProperties);
  AddVerb('Create Queue', CreateQueue);
  inherited;
end;

procedure TOraQueueAdminEditor.ReadQueueProperties;
begin
  TOraQueueAdmin(Component).ReadQueueProperties;
  if Self.Designer <> nil then
    Self.Designer.Modified;
end;

procedure TOraQueueAdminEditor.CreateQueue;
begin
  TOraQueueAdmin(Component).CreateQueue;
  ShowMessage('Queue created');
end;
{$ENDIF}

{$IFNDEF STD}
{$IFDEF DBTOOLS}
{ TOraPackageEditor }

 procedure TOraPackageEditor.InitVerbs;
 begin
   AddDBToolsVerbs([dbtEditDatabaseObject, dbtFindInDatabaseExplorer]);
 end;
{$ENDIF}
{$ENDIF}

{ TOraDSDesigner }

{$IFDEF OLDDESIGNER}
constructor TOraDSDesigner.Create(DataSet: TDataSet);
begin
  inherited Create(DataSet);

  FFieldsEditor := nil;
end;

destructor TOraDSDesigner.Destroy;
begin
  inherited;
end;
{$ENDIF}

procedure Register;
begin
  RegisterFieldClasses;

// Register property editors

  RegisterPropertyEditor(TypeInfo(string), TSmartQuery, 'KeyFields',
    TSmartQueryKeyFieldsEditor);
  RegisterPropertyEditor(TypeInfo(string), TOraDataSet, 'KeySequence',
    TOraDataSetKeySequenceEditor);

  RegisterPropertyEditor(TypeInfo(TCustomConnectDialog), TOraSession, 'ConnectDialog', TOraConnectDialogPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TOraQuery, 'SQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraQuery, 'SQLDelete', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraQuery, 'SQLInsert', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraQuery, 'SQLLock', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraQuery, 'SQLRecCount', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraQuery, 'SQLRefresh', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraQuery, 'SQLUpdate', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TSmartQuery, 'SQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TSmartQuery, 'SQLDelete', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TSmartQuery, 'SQLInsert', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TSmartQuery, 'SQLLock', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TSmartQuery, 'SQLRecCount', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TSmartQuery, 'SQLRefresh', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TSmartQuery, 'SQLUpdate', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TOraTable, 'SQL', nil);
  RegisterPropertyEditor(TypeInfo(TDAParams), TOraTable, 'Params', nil);  
  RegisterPropertyEditor(TypeInfo(TStrings), TOraStoredProc, 'SQL', nil);

  RegisterPropertyEditor(TypeInfo(TStrings), TOraSQL, 'SQL', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TOraScript, 'SQL', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TOraStoredProc, 'SQL', nil);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraStoredProc, 'SQLDelete', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraStoredProc, 'SQLInsert', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraStoredProc, 'SQLRecCount', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraStoredProc, 'SQLRefresh', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraStoredProc, 'SQLUpdate', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraStoredProc, 'SQLLock', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TOraUpdateSQL, 'InsertSQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraUpdateSQL, 'ModifySQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraUpdateSQL, 'DeleteSQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TOraUpdateSQL, 'RefreshSQL', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(string), TOraTable, 'TableName',
    TDATableNameEditor);
  RegisterPropertyEditor(TypeInfo(string), TOraTable, 'KeyFields',
    TOraTableKeyFieldsEditor);
{$IFNDEF STD}
  RegisterPropertyEditor(TypeInfo(string), TCustomOraPackage, 'PackageName',
    TOraPackageNameEditor);
  RegisterPropertyEditor(TypeInfo(String), TOraQueue, 'QueueName',
    TOraQueueNameEditor);
  RegisterPropertyEditor(TypeInfo(String), TOraQueueAdmin, 'QueueName',
    TOraQueueNameEditor);
  RegisterPropertyEditor(TypeInfo(String), TOraQueueTable, 'QueueTableName',
    TOraQueueTableNameEditor);
  RegisterPropertyEditor(TypeInfo(String), TOraQueueAdmin, 'QueueTableName',
    TOraQueueAdminQueueTableNameEditor);

  RegisterPropertyEditor(TypeInfo(Integer), TQueueMessageProperties, 'Delay',
    TQueueMessagePropertiesDelayEditor);
  RegisterPropertyEditor(TypeInfo(Integer), TQueueMessageProperties, 'Expiration',
    TQueueMessagePropertiesExpirationEditor);
  RegisterPropertyEditor(TypeInfo(Integer), TDequeueOptions, 'WaitTimeout',
    TDequeueOptionsWaitTimeoutEditor);
  RegisterPropertyEditor(TypeInfo(Integer), TOraQueueAdmin, 'RetentionTime',
    TOraQueueAdminRetentionTimeEditor);

  RegisterPropertyEditor(TypeInfo(Integer), TOraTrace, 'MaxTraceFileSize',
    TOraTraceMaxTraceFileSizeEditor);
{$ENDIF}    

{$IFDEF WIN32}
  RegisterPropertyEditor(TypeInfo(String), TOraSession, 'HomeName',
    TOraSessionHomeNameEditor);
{$ENDIF}
  RegisterPropertyEditor(TypeInfo(string), TOraSessionOptions, 'Charset',
    TOraSessionOptionsCharsetEditor);

  RegisterPropertyEditor(TypeInfo(Variant), TOraParam, 'Value', TVariantEditor);

// Obsolete properties
  RegisterPropertyEditor(TypeInfo(TRefreshMode), TOraDataSet, 'RefreshMode', nil);
  RegisterPropertyEditor(TypeInfo(Boolean), TOraDataSet, 'LocalConstraints', nil);
  RegisterPropertyEditor(TypeInfo(Boolean), TOraDataSet, 'ReturnParams', nil);
  RegisterPropertyEditor(TypeInfo(TOraDataSetOptionsDS), TOraDataSet, 'OptionsDS', nil);
  RegisterPropertyEditor(TypeInfo(TOracleHome), TOraSession, 'Home', nil);
  RegisterPropertyEditor(TypeInfo(Boolean), TOraStoredProc, 'ParamCheck', nil);
  RegisterPropertyEditor(TypeInfo(Boolean), TOraSessionOptions, 'NeverConnect', nil);
  RegisterPropertyEditor(TypeInfo(Boolean), TOraSessionOptions, 'Net', nil);  
  RegisterPropertyEditor(TypeInfo(Boolean), TSmartQuery, 'FullRefresh', nil);
  RegisterPropertyEditor(TypeInfo(Boolean), TOraDataSetOptions, 'KeepPrepared', nil);
  RegisterPropertyEditor(TypeInfo(Boolean), TOraSession, 'ConnectPrompt', nil);
{$IFNDEF STD}
  RegisterPropertyEditor(TypeInfo(TDAColumnDataType), TDPColumn, 'DataType', nil);
{$ENDIF}

{$IFNDEF WIN32}
  RegisterPropertyEditor(TypeInfo(String), TOraSession, 'HomeName', nil);
{$ENDIF}

// Register component editors
  DARegisterComponentEditor(TOraSession, TOraSessionEditor, TOraSessionEditorForm, TOraDesignUtils);
  DARegisterComponentEditor(TOraSQL, TOraSQLEditor, TOraSQLEditorForm, TOraDesignUtils);
  DARegisterComponentEditor(TOraQuery, TOraQueryEditor, TOraQueryEditorForm, TOraDesignUtils);
  DARegisterComponentEditor(TSmartQuery, TSmartQueryEditor, TSmartQueryEditorForm, TOraDesignUtils);
  DARegisterComponentEditor(TOraTable, TOraTableEditor, TOraTableEditorForm, TOraDesignUtils);
  DARegisterComponentEditor(TOraStoredProc, TOraStoredProcEditor, TOraStoredProcEditorForm, TOraDesignUtils);
  DARegisterComponentEditor(TOraUpdateSQL, TOraUpdateSQLEditor, TOraUpdateSQLEditorForm, TOraDesignUtils);
  DARegisterComponentEditor(TOraScript, TOraScriptEditor, TDAScriptEditorForm, TOraDesignUtils);
{$IFNDEF STD}
  DARegisterComponentEditor(TOraErrorHandler, TOraErrorHandlerEditor, TOraErrorHandlerEditorForm, TOraDesignUtils);
{$ENDIF}
{$IFNDEF FPC}
  DARegisterComponentEditor(TOraDataSource, TCRDataSourceEditor, nil, TOraDesignUtils);
{$ENDIF}
  // DAComponentEditor is needed to display session select dialog with Delphi 5
  DARegisterComponentEditor(TOraMetaData, TDAComponentEditor, nil, TOraDesignUtils);
{$IFNDEF STD}
{$IFDEF MSWINDOWS}
{$IFNDEF FPC}
  DARegisterComponentEditor(TOraAlerter, TDAComponentEditor, nil, TOraDesignUtils);
{$ENDIF}
{$ENDIF}
{$IFDEF DBTOOLS}
  DARegisterComponentEditor(TCustomOraPackage, TOraPackageEditor, nil, TOraDesignUtils);
{$ELSE}
  DARegisterComponentEditor(TCustomOraPackage, TDAComponentEditor, nil, TOraDesignUtils);
{$ENDIF}
  DARegisterComponentEditor(TOraTransaction, TDAComponentEditor, nil, TOraDesignUtils);
  DARegisterComponentEditor(TOraQueue, TDAComponentEditor, nil, TOraDesignUtils);
  DARegisterComponentEditor(TOraQueueTable, TOraQueueTableEditor, nil, TOraDesignUtils);
  DARegisterComponentEditor(TOraQueueAdmin, TOraQueueAdminEditor, nil, TOraDesignUtils);
  DARegisterComponentEditor(TOraTrace, TDAComponentEditor, nil, TOraDesignUtils);
{$ENDIF}
{$IFDEF DBTOOLS}
{$IFDEF WIN32}
//  TOraDesignUtils.DBToolsAvailable;
{$ENDIF}
{$ENDIF}
{$IFNDEF FPC}
  Menu.AddItems({$IFDEF CLR}WinUtils{$ELSE}SysInit{$ENDIF}.HInstance);
{$ENDIF}
end;

{$IFDEF VER6P}
{$IFNDEF FPC}
var
  Notificator: IDesignNotification;
{$ENDIF}
{$ENDIF}

initialization
{$IFDEF DBTOOLS}
  if not Assigned(DBTools) then
    DBTools := TDBTools.Create;
{$ENDIF}

{$IFDEF VER6P}
{$IFNDEF FPC}
  Notificator := TOraDesignNotification.Create;
  RegisterDesignNotification(Notificator);
{$ENDIF}
{$ENDIF}

finalization
{$IFDEF VER6P}
{$IFNDEF FPC}
  UnRegisterDesignNotification(Notificator);
  Notificator := nil;
{$ENDIF}
{$ENDIF}

end.
