
//////////////////////////////////////////////////
//  Oracle Data Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  OraParamArrayValue Editor
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I Odac.inc}

unit OraParamArrayValueEditor;
{$ENDIF}

interface
uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, ExtCtrls, Grids,
{$IFDEF VER6P}
  Variants,
{$ENDIF}
{$IFDEF FPC}
  LResources,
{$ENDIF}
  DB, CREditor, CRDesignUtils, DAEditor, DADesignUtils;

type

  TTableParamValueEditor = class(TDAEditorForm)
    sgValues: TStringGrid;
    btNull: TBitBtn;
    btEdit: TBitBtn;
    Panel1: TPanel;
    procedure lvValuesChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure FormResize(Sender: TObject);
    procedure sgValuesDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure sgValuesSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure sgValuesTopLeftChanged(Sender: TObject);
    procedure sgValuesEnter(Sender: TObject);
    procedure btOkEnter(Sender: TObject);
    procedure btCancelEnter(Sender: TObject);
    procedure sgValuesSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: string);
    procedure btNullClick(Sender: TObject);
    procedure btEditClick(Sender: TObject);
  protected
    FLastEditText: string;
    FLastEditRow: integer;
    FDataType: word;
    FNulls: array of boolean;

    procedure DoSave; override;

    function GetCount: integer;
    procedure SetCount(const Value: integer);
    function GetValue(Index: integer): variant;
    procedure SetValue(Index: integer; const Value: variant);

    function GetComponent: TComponent; override;
    procedure SetComponent(Value: TComponent); override;
    procedure DoInit; override;
    procedure DoFinish; override;

    procedure AdjustButtons(GridRow: integer);
    procedure HideButtons;
    procedure Validate;
  public
    constructor Create(Owner: TComponent; CRDesignUtilsClass: TCRDesignUtilsClass); override;
    destructor Destroy; override;
    procedure Clear;

    property Value[Index: integer]: variant read GetValue write SetValue;
    property Count: integer read GetCount write SetCount;
    property DataType: word read FDataType write FDataType;
  end;

implementation

uses
  MemData, CRParser, DAParamValueEditor, OraParamsFrame;

{$IFNDEF FPC}
{$R OraParamArrayValueEditor.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

function TTableParamValueEditor.GetComponent: TComponent;
begin
  Result := nil;
end;

procedure TTableParamValueEditor.SetComponent(Value: TComponent);
begin
end;

function TTableParamValueEditor.GetValue(Index: integer): variant;
begin
  if FNulls[Index] then
    Result := Null
  else
    Result := VarAsType(sgValues.Cells[1, Index], DataType);
end;

procedure TTableParamValueEditor.SetValue(Index: integer;
  const Value: variant);
begin
  with sgValues do
    if VarIsNull(Value) then begin
      FNulls[Index] := True;
      Cells[1, Index] := '';
    end
    else begin
      FNulls[Index] := False;
      Cells[1, Index] := Value;
    end;
end;

procedure TTableParamValueEditor.DoInit;
begin
  Modified := False;
  AdjustButtons(sgValues.Row);
  sgValues.SetFocus;
end;

procedure TTableParamValueEditor.DoSave;
begin
  Assert(ModalResult <> mrCancel);
  Validate;
  inherited;
end;

function TTableParamValueEditor.GetCount: integer;
begin
  Result := Length(FNulls);
end;

procedure TTableParamValueEditor.SetCount(const Value: integer);
var
  i, n: integer;
  s: string;
begin
  s := '';
  for i := 0 to Length(IntToStr(Value)) do
    s := s + '8';
  with sgValues do
    ColWidths[0] := Canvas.TextWidth(s);
  if Value > 0 then
    sgValues.RowCount := Value
  else
    sgValues.RowCount := 1;
  n := GetCount;
  SetLength(FNulls, Value);
  for i := n to Value - 1 do
    FNulls[i] := True;
  for i := 0 to Value - 1 do
    sgValues.Cells[0, i] := IntToStr(i + 1);
  FormResize(nil);
end;

procedure TTableParamValueEditor.Clear;
begin
  SetCount(0);
end;

procedure TTableParamValueEditor.lvValuesChange(Sender: TObject;
  Item: TListItem; Change: TItemChange);
begin
  if Item.Checked then
    Item.Caption := '';
end;

procedure TTableParamValueEditor.DoFinish;
begin
  inherited;
end;

constructor TTableParamValueEditor.Create(Owner: TComponent;
  CRDesignUtilsClass: TCRDesignUtilsClass);
begin
  inherited;
  FLastEditRow := -1;
end;

procedure TTableParamValueEditor.FormResize(Sender: TObject);
begin
  with sgValues do begin
    ColWidths[1] := ClientWidth - ColWidths[0] - 38;
    btNull.Left := Left + ClientWidth - 32;
    btEdit.Left := Left + ClientWidth - 16;
    AdjustButtons(Row);
  end;
end;

procedure TTableParamValueEditor.sgValuesDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with sgValues, Canvas do
    if (ACol = 1) and FNulls[ARow] then begin
      TextRect(Rect, Rect.Left + 2, Rect.Top + 2, '(null)');
    end;
end;

procedure TTableParamValueEditor.sgValuesSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  Validate;
  AdjustButtons(ARow);
end;

procedure TTableParamValueEditor.sgValuesTopLeftChanged(Sender: TObject);
begin
  AdjustButtons(sgValues.Row);
end;

procedure TTableParamValueEditor.AdjustButtons(GridRow: integer);
var
  Y: integer;
begin
  if (sgValues.RowCount > Count) or (GridRow < 0) or (GridRow >= sgValues.RowCount) then
    Exit;
  with sgValues do begin
    with CellRect(1, GridRow) do begin
      if (Right <= Left) or (Bottom <= Top) or (Bottom >= ClientHeight) then
        Y := -1
      else
        Y := Top;
    end;
    if Y >= 0 then begin
      btNull.Top := Top + Y + 2;
      btEdit.Top := btNull.Top;
      btEdit.Visible := True;
      btNull.Visible := not FNulls[GridRow];
    end
    else
      HideButtons;
  end;
end;

procedure TTableParamValueEditor.HideButtons;
begin
  btEdit.Visible := False;
  btNull.Visible := False;
end;

procedure TTableParamValueEditor.sgValuesEnter(Sender: TObject);
begin
  AdjustButtons(sgValues.Row);
end;

procedure TTableParamValueEditor.btOkEnter(Sender: TObject);
begin
  HideButtons;
  inherited;
end;

procedure TTableParamValueEditor.btCancelEnter(Sender: TObject);
begin
  HideButtons;
  inherited;
end;

procedure TTableParamValueEditor.sgValuesSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: string);
begin
  FLastEditText := Value;
  FLastEditRow := Arow;
  if FNulls[ARow] and (Value <> '') then begin
    FNulls[ARow] := False;
    AdjustButtons(ARow);
  end;
  Modified := True;
end;

procedure TTableParamValueEditor.btNullClick(Sender: TObject);
begin
  with sgValues do begin
    Cells[1, Row] := '';
    FNulls[Row] := True;
    AdjustButtons(Row);
  end;
end;

destructor TTableParamValueEditor.Destroy;
begin
  Clear;
  inherited;
end;

procedure TTableParamValueEditor.btEditClick(Sender: TObject);
begin
  with TDAParamValueEditor.Create(Self, TDAEditorForm(TOraParamsFrame(Owner).Owner).DADesignUtilsClass) do
    try
      Value := sgValues.Cells[1, sgValues.Row];
      ShowModal;
      if ModalResult = mrOK then begin
        if sgValues.Cells[1, sgValues.Row] <> Value then begin
          Modified := True;
          sgValues.Cells[1, sgValues.Row] := Value;
          sgValuesSetEditText(nil, 1, sgValues.Row, Value);
          Validate;
        end;
      end;
    finally
      Free;
    end;
end;

procedure TTableParamValueEditor.Validate;
begin
  if (ModalResult <> mrCancel) and (FLastEditRow >= 0) then
    try
      if not FNulls[FLastEditRow] then
        VarAsType(FLastEditText, DataType);
      FLastEditRow := -1;
    except
      sgValues.Row := FLastEditRow;
      sgValues.SetFocus;
      raise;
    end;
end;

end.
