
//////////////////////////////////////////////////
//  DB Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  ODAC Params Frame
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I Odac.inc}

unit OraParamsFrame;
{$ENDIF}

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
{$IFDEF FPC}
  LResources,
{$ENDIF}
  Classes, SysUtils, DB,
  CRFrame, CRTabEditor, DAParamsFrame, {$IFNDEF FPC}MemDS{$ELSE}MemDataSet{$ENDIF},
  DBAccess, Ora;

type
  TOraParamsFrame = class(TDAParamsFrame)
    cbTable: TCheckBox;
    lbPLSQLTable: TLabel;
    lbLength: TLabel;
    edLength: TEdit;
    procedure cbTableClick(Sender: TObject);
    procedure bEdValueClick(Sender: TObject);
  protected
    procedure UpdateControlsState; override;
    procedure ItemToControls(Item: TCollectionItem); override;
    procedure StoreItemValue(Item: TCollectionItem); override;
    procedure ControlsToItem(Item: TCollectionItem); override;
    procedure InitItems; override;

    function TableParamToText(Param: TOraParam): string;
  public
   constructor Create(AOwner: TComponent); override;
  end;

implementation

{$IFNDEF FPC}
{$R OraParamsFrame.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  CRColFrame,
  {$IFDEF VER6P}
  Variants,
  {$ENDIF}
  OraParamArrayValueEditor, DAEditor;

{ TOraParamsFrame }

constructor TOraParamsFrame.Create(AOwner: TComponent);
begin
  inherited;
  AddDataType('String',        ftString,             False, True,  True,  '');
  AddDataType('WideString',    ftWideString,         False, True,  True,  '');
  AddDataType('Integer',       ftInteger,            True,  True,  False, '0');
  AddDataType('Float',         ftFloat,              True,  True,  False, '0');
  AddDataType('Number',        TFieldType(ftNumber), True,  True,  False, '');

  AddDataType('Date',          ftDateTime,           True,  True,   False, '');
  AddDataType('FixedChar',     ftFixedChar,          True,  True,   True, '');
  AddDataType('FixedWideChar', TFieldType(ftFixedWideChar), True,  True,   True, '');
  AddDataType('Memo',          ftMemo,               True,  True,   False, '');
{$IFDEF VER10P}
  AddDataType('WideMemo',      ftWideMemo,           True,  True,   False, '');
{$ENDIF}
  AddDataType('Blob',          ftBlob,               True,  True,   False, '');
  AddDataType('OraClob',       ftOraClob,            True,  True,   False, '');
  AddDataType('OraBlob',       ftOraBlob,            True,  True,   False, '');
  AddDataType('Cursor',        ftCursor,             True,  False,  False, '');
  AddDataType('Object',        ftObject,             True,  False,  False, '');
  AddDataType('XML',           TFieldType(ftXML),    True,  True,   False, '');
  AddDataType('Reference',     ftReference,          True,  False,  False, '');
  AddDataType('Array',         ftArray,              True,  False,  False, '');
  AddDataType('Table',         ftTable,              True,  True,   False, '');
  AddDataType('BFile',         TFieldType(ftBFile),  False, False,  False, '');
  AddDataType('Boolean',       ftBoolean,            True,  True,   False, '');
{$IFDEF VER6P}
{$IFNDEF FPC}
  AddDataType('Timestamp',     TFieldType(ftTimeStamp), True,  True,   False, '');
{$ENDIF}
{$ENDIF}
  AddDataType('OraTimestamp',     TFieldType(ftOraTimeStamp),    True,  True,   False, '');
  AddDataType('OraTimestampTZ',   TFieldType(ftOraTimeStampTZ),  True,  True,  False, '');
  AddDataType('OraTimestampLTZ',  TFieldType(ftOraTimeStampLTZ), True,  True,  False, '');
  AddDataType('IntervalYM',       TFieldType(ftIntervalYM), True,  True,  False, '');
  AddDataType('IntervalDS',       TFieldType(ftIntervalDS), True,  True,  False, '');
  AddDataType('VarBytes',      ftVarBytes,           True,  True,   False, '');

  AddParamType('IN', ptInput);
  AddParamType('OUT', ptOutput);
  AddParamType('IN/OUT', ptInputOutput);
end;

procedure TOraParamsFrame.cbTableClick(Sender: TObject);
begin
  if not FInStoreItem and not FInSelectItem and (lbItemName.ItemIndex <> -1) then begin
    Modified := True;
    StoreItem;
    SelectItem;
  end;
end;

procedure TOraParamsFrame.UpdateControlsState;
var
  i: integer;
  Param: TOraParam;
  ParamCollection: TCollection;
begin
  cbTable.Enabled := (cbDataType.ItemIndex <> - 1) and (FDataTypeInfos[cbDataType.ItemIndex].DataType in
    [ftString, ftFixedChar, ftWideString, ftInteger, ftFloat, ftDateTime]);
  if not cbTable.Enabled and cbTable.Checked then
    cbTable.Checked := False;
  edLength.Enabled := cbTable.Checked;

  if lbItemName.ItemIndex <> -1 then begin
    ParamCollection := Items;
    // Items can be nil in Lazarus for Mac OS X
    if ParamCollection <> nil then begin
      Param := TOraParam(ParamCollection.Items[lbItemName.ItemIndex]);
      if Param.Table and cbTable.Checked and (Param.DataType <> FDataTypeInfos[cbDataType.ItemIndex].DataType) then begin
        for i := 1 to Param.Length do
          Param.ItemValue[i] := Null;
        StoreItem;
        SelectItem;
      end;
    end;
  end;   

  inherited;
  if cbTable.Checked then begin
    edValue.Enabled := False;
    cbNullValue.Enabled := False;
    edValue.Color := clBtnFace;
  end;
end;

procedure TOraParamsFrame.InitItems;
begin
  inherited;
end;

procedure TOraParamsFrame.ItemToControls(Item: TCollectionItem);
var
  Param: TOraParam;
begin
  Param := TOraParam(Item);
  cbTable.Checked := Param.Table;
  edLength.Enabled := Param.Table;
  if Param.Table then begin
    cbNullValue.Enabled := False;
    edValue.Enabled := False;
    edLength.Text := IntToStr(Param.Length);
  end;
  inherited;
  if Param.Table then begin
    edValue.Enabled := False;
    edValue.Text := TableParamToText(Param);
  end;
end;

procedure TOraParamsFrame.StoreItemValue(Item: TCollectionItem);
begin
  if not TOraParam(Item).Table then
    inherited
end;

procedure TOraParamsFrame.ControlsToItem(Item: TCollectionItem);
var
  Param: TOraParam;
  BadControl: TWinControl;
begin
  BadControl := nil;
  try
    Param := TOraParam(Item);
    if Param.Table and not cbTable.Checked then
      Param.Table := False;
    inherited;
    BadControl := cbTable;
    if not Param.Table and cbTable.Checked then
      Param.Table := True;
    BadControl := edLength;
    if Param.Table then
      if Trim(edLength.Text) = '' then
        Param.Length := 1
      else
        Param.Length := StrToInt(edLength.Text);
  except
    if Page.PageControl.ActivePage = Page then begin
      lbItemName.ItemIndex := FOldItemIndex;

      if BadControl <> nil then begin
        BadControl.SetFocus;
        if BadControl is TCustomEdit then
          TCustomEdit(BadControl).SelectAll;
      end;
    end;
    raise;
  end;
end;

procedure TOraParamsFrame.bEdValueClick(Sender: TObject);
var
  Param: TOraParam;
  i: integer;
begin
  if not cbTable.Checked then
    inherited
  else
  with TTableParamValueEditor.Create(Self, TDAEditorForm(Owner).DADesignUtilsClass) do
    try
      Param := TOraParam(Items.Items[lbItemName.ItemIndex]);
      DataType := FieldTypeVarMap[Param.DataType];
      Clear;
      Count := Param.Length;
      for i := 1 to Param.Length do
        if Param.ItemIsNull[i] then
          Value[i - 1] := Null
        else
          Value[i - 1] := Param.ItemValue[i];
      ShowModal;
      if ModalResult = mrOK then begin

        for i := 1 to Param.Length do
          if VarIsNull(Value[i - 1]) then
            Param.ItemValue[i] := Null
          else
            Param.ItemText[i] := Value[i - 1];
        StoreItem;
        SelectItem;
      end;
    finally
      Free;
    end;
end;

function TOraParamsFrame.TableParamToText(Param: TOraParam): string;
var
  i: integer;
  s: string;
begin
  s := '';
  for i := 1 to Param.Length do begin
    if i > 1 then
      s := s + '; ';
    if Param.ItemIsNull[i] then
      s := s + '(null)'
    else
      if Param.DataType in [ftString, ftDateTime] then
        s := s + '''' + Param.ItemText[i] + ''''
      else
        s := s + Param.ItemText[i];
  end;
  Result := s;
end;

end.
