
//////////////////////////////////////////////////
//  DB Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  InterBase Stored Proc Call Generator Frame
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I Odac.inc}

unit OraSPCallFrame;
{$ENDIF}

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls,
{$IFDEF FPC}
  LResources,
{$ENDIF}
  Classes, SysUtils, CRTypes, DBAccess,
  CRFrame, CRTabEditor, DASPCallFrame, CREditor;

type
  TOraSPCallFrame = class(TDASPCallFrame)
    edOverload: TEdit;
    lblOverload: TLabel;
    cbAllProcs: TCheckBox;
    lblOverload2: TLabel;
    edOverload2: TEdit;
    btCreateSQL: TSpeedButton;
    procedure edOverloadChange(Sender: TObject);
    procedure edOverloadExit(Sender: TObject);
    procedure cbAllProcsClick(Sender: TObject);
    procedure btCreateSQLClick(Sender: TObject);
  published
    udOverload: TUpDown;
    udOverload2: TUpDown;
  protected
    function GetOverload: integer;
    procedure SetOverload(const Value: integer);
    procedure UpdateStoredProcSelection; override;
    procedure ParseSPName;
    function ShowAllProc: boolean; override;
  public
    function GetSPName: string; override;
    procedure SetSPName(const Value: string); override;
  end;

implementation

{$IFNDEF FPC}
{$R OraSPCallFrame.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  CRFunctions, Ora, OraClasses, DASQLFrame, DAConsts;

{ TOraSPCallFrame }

function TOraSPCallFrame.GetOverload: integer;
begin
  case Mode of
    spSQL, spQuery, spQuerySP:
      if not TryStrToInt(edOverload2.Text, Result) then
        Result := 0;
    spSQLSP:
      if not TryStrToInt(edOverload.Text, Result) then
        Result := 0;
  end;
end;

procedure TOraSPCallFrame.SetOverload(const Value: integer);
begin
  case Mode of
    spSQL, spQuery, spQuerySP:
      edOverload2.Text := IntToStr(Value);
    spSQLSP:
      edOverload.Text := IntToStr(Value);
  end;
end;

procedure TOraSPCallFrame.UpdateStoredProcSelection;
var
  SPName: string;
  SPOverload: integer;
begin
  if TOraUtils.ParseSPName(inherited GetSPName, SPName, SPOverload) then
    SetOverload(SPOverload)
  else
    SetOverload(0);

  inherited;
end;

procedure TOraSPCallFrame.ParseSPName;
var
  ProcName: string;
  ProcOverload: integer;
begin
  ProcName := inherited GetSPName;

  if TOraUtils.ParseSPName(ProcName, ProcName, ProcOverload) then
    SetOverload(ProcOverload);
end;

function TOraSPCallFrame.ShowAllProc: boolean;
begin
  Result := cbAllProcs.Checked;
end;

function TOraSPCallFrame.GetSPName: string;
var
  SPOverload: integer;
begin
  SPOverload := GetOverload;

  Result := TOraUtils.GetFinalSPName(inherited GetSPName, SPOverload);

  SetSPName(Result);
end;

procedure TOraSPCallFrame.SetSPName(const Value: string);
begin
  inherited SetSPName(Value);

  ParseSPName;
end;

procedure TOraSPCallFrame.edOverloadChange(Sender: TObject);
var
  SPName: string;
  SPOverload: integer;
begin
  if TOraUtils.ParseSPName(inherited GetSPName, SPName, SPOverload) then begin
    SPOverload := GetOverload;
    if SPOverload > 0 then
      inherited SetSPName(SPName + ':' + IntToStr(SPOverload))
    else
      inherited SetSPName(SPName);
  end;    
end;

procedure TOraSPCallFrame.edOverloadExit(Sender: TObject);
begin
  if (edOverload.Text <> '') and
    (IntToStr((Editor.LocalComponent as TOraStoredProc).Overload) <> edOverload.Text)
  then
    CreateProcedureCall;
end;

procedure TOraSPCallFrame.cbAllProcsClick(Sender: TObject);
begin
  FListGot := False;
end;

procedure TOraSPCallFrame.btCreateSQLClick(Sender: TObject);
begin
  CreateProcedureCall;
end;

end.
