
//////////////////////////////////////////////////
//  Oracle Data Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  OraSession Editor
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I Odac.inc}

unit OraSessionEditor;
{$ENDIF}
interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, DacVcl, Buttons,
{$IFNDEF FPC}
  Mask,
{$ENDIF}
{$IFDEF DBTOOLS}
  DBToolsClient,
{$ENDIF}
{$IFDEF FPC}
  LResources,
{$ENDIF}
  CRTypes, DBAccess, CREditor, DAConnectionEditor, Ora, OraCall;

type
  TOraSessionEditorUtils = class(TObject)
  public
    class procedure DivConnStr(edServer, edSID, edPort, edHost: TComboBox; cbServiceName: TCheckBox);
  {$IFDEF DBTOOLS}
    class procedure ExistingConnectionChange(const ConnectionName: string; const Connection: TOraSession);
  {$ENDIF}
  end;

  TOraSessionEditorForm = class(TDAConnectionEditorForm)
    edSID: TComboBox;
    edHome: TComboBox;
    lbHome: TLabel;
    cbDirect: TCheckBox;
    cbDetail: TCheckBox;
    edHost: TComboBox;
    edPort: TComboBox;
    cbExisting: TComboBox;
    lbExisting: TLabel;
    lbPort: TLabel;
    cbServiceName: TCheckBox;
    procedure cbDirectClick(Sender: TObject);
    procedure cbDetailClick(Sender: TObject);
    procedure edHostChange(Sender: TObject);
    procedure edHomeChange(Sender: TObject);
    procedure cbExistingChange(Sender: TObject);
    procedure cbServiceNameClick(Sender: TObject);
    procedure edPortSIDHostDropDown(Sender: TObject);
  private
    procedure DivConnStr;
    procedure MakeConnStr;
    function GetConnection: TOraSession;
    function GetLocalConnection: TOraSession;
    function GetHomeName: string;
  protected
  {$IFDEF DBTOOLS}
    function GetExistingConnectionComboBox: TComboBox; override;
  {$IFNDEF NET}
    function GetConnectionCondition: string; override;
  {$ENDIF}
  {$ENDIF}

    procedure Resize; override;

    procedure DoInit; override;
    procedure FillInfo(InfoConnection: TCustomDAConnection); override;
    procedure AssignHomeName(const Value: string);
    procedure AssignDirect(const Value: boolean);
    procedure UpdateControls;
    procedure DoSaveConnection; override;

    procedure ConnToControls; override;

  public
    property Connection: TOraSession read GetConnection;
    property LocalConnection: TOraSession read GetLocalConnection;
  end;

var
  OraSessionEditorForm: TOraSessionEditorForm;

implementation

{$IFNDEF FPC}
{$IFDEF IDE}
{$R *.dfm}
{$ELSE}
{$R OraSessionEditor.dfm}
{$ENDIF}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
{$IFDEF VER6P}
  Variants,
{$ENDIF}
  OdacVcl,
{$IFDEF DBTOOLS}
  OraDesign,
{$ENDIF}
  OraDesignUtils, MemData, OraClasses;

{ TOraSessionEditorForm }

procedure TOraSessionEditorForm.Resize;
begin
  inherited;

  if cbServiceName <> nil then begin
  {$IFDEF DBTOOLS}
    cbServiceName.Top := cbDetail.Top;
    cbServiceName.Left := 256;
  {$ELSE}
    cbServiceName.Top := cbDetail.Top + (cbDetail.Top - cbDirect.Top);
    cbServiceName.Left := cbDetail.Left;
  {$ENDIF}
  end;
end;

procedure TOraSessionEditorForm.DoInit;
var
  i: integer;
begin
  if (OracleHomes.Default <> nil) and (OracleHomes.Default.Name <> '') then
    edHome.Items[0] := edHome.Items[0] + ' [' + OracleHomes.Default.Name + ']';
  for i := 0 to OracleHomes.Count - 1 do
    edHome.Items.Add(OracleHomes[i].Name);

  inherited;

  if cbDetail.Checked then
    DivConnStr;

  lbVersion.Caption := ODACVersion + ' ';
  UpdateVersionPosition;

{$IFDEF BETA}
  lbEdition.Caption := 'Beta';
  lbEdition.Font.Color := clRed;
{$ELSE}
{$IFDEF RC}
  lbEdition.Caption := 'Release Candidate';
  lbEdition.Font.Color := clGreen;
{$ELSE}
{$IFDEF TRIAL}
  lbEdition.Caption := 'Trial Edition';
{$ELSE} //ORIGINAL
{$IFDEF STD}
  lbEdition.Caption := 'Standard Edition';
{$ELSE}
  lbEdition.Caption := 'Professional Edition';
{$ENDIF}
{$ENDIF}
{$ENDIF}
{$ENDIF}

{$IFDEF UNIX}
  edPort.OnGetItems := edPortSIDHostDropDown;
  edSID.OnGetItems := edPortSIDHostDropDown;
  edHost.OnGetItems := edPortSIDHostDropDown;
{$ELSE}
  edPort.OnDropDown := edPortSIDHostDropDown;
  edSID.OnDropDown := edPortSIDHostDropDown;
  edHost.OnDropDown := edPortSIDHostDropDown;
{$ENDIF}

  if Connection.Owner <> nil then
    Caption := Connection.Owner.Name + '.' + Connection.Name;

  ShowState;

{$IFNDEF NET}
  cbDirect.Visible := False;
  cbDetail.Visible := False;
  cbServiceName.Visible := False;
{$ENDIF}
{$IFDEF MSWINDOWS}
  imPeng.Visible := False;
{$ENDIF}
{$IFDEF CLR}
  lbHome.Enabled := False;
  edHome.Enabled := False;
{$ENDIF}
{$IFNDEF MSWINDOWS}
  imPeng.BringToFront;
  lbHome.Enabled := False;
  edHome.Enabled := False;
{$ENDIF}
{$IFDEF MSWINDOWS}
{$IFNDEF FPC}
  if not DADesignUtilsClass.DBToolsAvailable and cbExisting.Visible then
{$ENDIF}
{$ENDIF}
  begin
    lbExisting.Visible := False;
    cbExisting.Visible := False;
    Panel.Top := Panel.Top - 33;
    Resize;
  end;
end;

procedure TOraSessionEditorForm.cbDirectClick(Sender: TObject);
begin
  try
    AssignDirect(cbDirect.Checked);

  {$IFDEF DBTOOLS}
    ChooseExistingConnection;
  {$ENDIF}

  finally
    UpdateControls;
  end;
end;

procedure TOraSessionEditorForm.MakeConnStr;
var
  ServerInfo: TDirectServerInfo;
begin
  ServerInfo := TDirectServerInfo.Create;
  try
    ServerInfo.Host := edHost.Text;
    ServerInfo.Port := edPort.Text;
    if cbServiceName.Checked then
      ServerInfo.ServiceName := edSID.Text
    else
      ServerInfo.SID := edSID.Text;

    edServer.Text := ServerInfo.GetServerInfo;
  finally
    ServerInfo.Free;
  end;

  edServerChange(nil);
end;

procedure TOraSessionEditorForm.DivConnStr;
begin
  TOraSessionEditorUtils.DivConnStr(edServer, edSID, edPort, edHost, cbServiceName);
end;

procedure TOraSessionEditorForm.cbDetailClick(Sender: TObject);
begin
  if cbDetail.Checked then
    DivConnStr;

  UpdateControls;
end;

procedure TOraSessionEditorForm.cbServiceNameClick(Sender: TObject);
begin
  MakeConnStr;
  UpdateControls;
end;

procedure TOraSessionEditorForm.edPortSIDHostDropDown(Sender: TObject);
begin
  edServerDropDown(Sender);
  DivConnStr;
end;

function TOraSessionEditorForm.GetConnection: TOraSession;
begin
  Result := FConnection as TOraSession;
end;

function TOraSessionEditorForm.GetLocalConnection: TOraSession;
begin
  Result := FLocalConnection as TOraSession;
end;

function TOraSessionEditorForm.GetHomeName: string;
begin
  if edHome.Text = edHome.Items[0] then
    Result := ''
  else
    Result := edHome.Text;
end;

procedure TOraSessionEditorForm.ConnToControls;
var
  i: integer;
begin
  if LocalConnection.HomeName = '' then
    edHome.ItemIndex := 0
  else begin
    for i := 0 to OracleHomes.Count - 1 do
      if AnsiCompareText(OracleHomes[i].Name, LocalConnection.HomeName) = 0 then begin
        edHome.ItemIndex := i + 1;
        break;
      end;
  end;
  if edHome.ItemIndex < 0 then
    edHome.Text := LocalConnection.HomeName;
  cbDirect.Checked := LocalConnection.Options.Direct;

  UpdateControls;

  inherited;
end;

procedure TOraSessionEditorForm.FillInfo(InfoConnection: TCustomDAConnection);
var
  OraInfoSession: TOraSession;
  OCIStatus: string;
  OCIVer: string;
  OCIFunc: string;
  OracleVer: string;
  St: string;
  i: integer;
  OldDebug: boolean;
begin
  OraInfoSession := TOraSession(InfoConnection);

  meInfo.Lines.BeginUpdate;
  if not OCILite then
    try
      if OraInfoSession.Connected then begin
        OldDebug := OraInfoSession.SQL.Debug;
        OraInfoSession.SQL.Debug := False;
        try
          OracleVer := OraInfoSession.ExecSQL(
            'begin SELECT Product || Version INTO :Result FROM SYS.PRODUCT_COMPONENT_VERSION ' +
            'WHERE Upper(Product) LIKE ''%ORACLE%''; end;', [Null]);
        finally
          OraInfoSession.SQL.Debug := OldDebug;
        end;
      end;

      if not OraInfoSession.Options.Direct then begin
        if (TOraSession(InfoConnection).Home = nil) or
           (not TOraSession(InfoConnection).Home.Inited)
        then begin
          OCIStatus := '(not loaded)';
          try
            OraInfoSession.HomeName := GetHomeName;
            OraInfoSession.Home.Init;
          except // CR18549
          end;
        end
        else
          OCIStatus := '';

        if OraInfoSession.Home <> nil then begin
          if OraInfoSession.Home.OCIVersionSt = '' then
            OCIVer := ''
          else
            OCIVer := 'Version ' + OraInfoSession.Home.OCIVersionSt;

          if OraInfoSession.OCICallStyle = OCI73 then
            OCIFunc := 'v7'
          else
            if OraInfoSession.OCICallStyle = OCI80 then
              OCIFunc := 'v8'
            else
              OCIFunc := '';

          St := 'OCI: ' + OCIVer + '  ' + OCIStatus + #13#10 +
            'DLL: ' + OraInfoSession.Home.OCIDLL + #13#10;

          for i := 0 to OracleHomes.Count - 1 do
            if OracleHomes[i].Path <> '' then
              St := St + 'Home [' + OracleHomes[i].Name + ']: ' + OracleHomes[i].Path + #13#10;

          if OracleHomes.Default <> nil then
            St := St + 'Default Home [' + OracleHomes.Default.Name + ']: ' + OracleHomes.Default.Path + #13#10;

          St := St + 'Current Home: ' + OraInfoSession.Home.Path + #13#10;

          St := St + 'Call style: ' + OCIFunc + #13#10 +
            'Oracle: ' + OracleVer;
        end;
      end
      else begin
        St := 'Oracle: ' + OracleVer;
      end;

      SetMemoText(meInfo, St);
    finally
      meInfo.Lines.EndUpdate;
    end;
end;

procedure TOraSessionEditorForm.AssignHomeName(const Value: string);
begin
  if LocalConnection.HomeName <> Value then begin
    PerformDisconnect;
    LocalConnection.HomeName := Value;
    Modified := True;
  end;
end;

procedure TOraSessionEditorForm.AssignDirect(const Value: boolean);
begin
  if LocalConnection.Options.Direct <> Value then begin
    PerformDisconnect;
    LocalConnection.Options.Direct := Value;
    Modified := True;
  end;
end;

procedure TOraSessionEditorForm.UpdateControls;
begin
{$IFDEF MSWINDOWS}
  lbHome.Enabled := (not LocalConnection.Options.Direct) or (cbDetail.Enabled and cbDetail.Checked);
  edHome.Enabled := not LocalConnection.Options.Direct;
{$ENDIF}
  cbDetail.Enabled := LocalConnection.Options.Direct;
  cbServiceName.Enabled := cbDetail.Enabled and cbDetail.Checked;

  if not cbDirect.Checked then begin
    cbDetail.Checked := False;
    cbServiceName.Checked := False;
  end;

  if not cbDetail.Checked then begin
    lbServer.Caption := 'Server';
    lbHome.Caption := 'Home';
    cbServiceName.Checked := False;
  end
  else begin
    lbServer.Caption := 'Host';
    if cbServiceName.Checked then
      lbHome.Caption := 'Service Name'
    else
      lbHome.Caption := 'SID';
  end;

  edServer.Visible := not cbDetail.Checked;
  edHome.Visible   := not cbDetail.Checked;
  edHost.Visible   := cbDetail.Checked;
  lbPort.Visible   := cbDetail.Checked;
  edPort.Visible   := cbDetail.Checked;
  edSID.Visible    := cbDetail.Checked;
end;

procedure TOraSessionEditorForm.DoSaveConnection;
begin
  inherited;
  Connection.HomeName := LocalConnection.HomeName;
  Connection.Options.Direct := LocalConnection.Options.Direct;
  Connection.Options.UseUnicode := LocalConnection.Options.UseUnicode;
  Connection.ConnectMode := LocalConnection.ConnectMode;
end;

procedure TOraSessionEditorForm.edHostChange(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  try
    MakeConnStr;
  finally
    ShowState;
  end;
end;

procedure TOraSessionEditorForm.edHomeChange(Sender: TObject);
begin
  if FInDoInit then
    Exit;

  try
    AssignHomeName(GetHomeName);
  finally
    ShowState;
  end;
end;

{$IFDEF DBTOOLS}
function TOraSessionEditorForm.GetExistingConnectionComboBox: TComboBox;
begin
  Result := cbExisting;
end;

{$IFNDEF NET}
function TOraSessionEditorForm.GetConnectionCondition: string;
begin
  Result := 'Direct=False';
end;
{$ENDIF}
{$ENDIF}

procedure TOraSessionEditorForm.cbExistingChange(Sender: TObject);
{$IFDEF DBTOOLS}
begin
  if FInDoInit or (cbExisting.ItemIndex < 0) then
    Exit;
  FInExistingChange := True;
  try
    TOraSessionEditorUtils.ExistingConnectionChange(cbExisting.Text, LocalConnection);
  finally
    ConnToControls;
    FInExistingChange := False;
  end;
end;  
{$ELSE}
begin
end;
{$ENDIF}

{ TOraSessionEditorUtils }

class procedure TOraSessionEditorUtils.DivConnStr(edServer, edSID, edPort, edHost: TComboBox; cbServiceName: TCheckBox);
var
  i: integer;
  Service: string;
  Port: string;
  Host: string;
  UseServiceName: boolean;
  ServerInfo: TDirectServerInfo;
begin
  ServerInfo := TDirectServerInfo.Create;
  try
    ServerInfo.SetServerInfo(edServer.Text);

    Host := ServerInfo.Host;
    Port := ServerInfo.Port;
    if ServerInfo.ServiceName <> '' then begin
      UseServiceName := True;
      Service := ServerInfo.ServiceName;
    end
    else begin
      UseServiceName := False;
      Service := ServerInfo.SID;
    end;
  finally
    ServerInfo.Free;
  end;

  edSID.Text := Service;
  edPort.Text := Port;
  edHost.Text := Host;
  cbServiceName.Checked := UseServiceName;

  edSID.Items.Clear;
  edPort.Items.Clear;
  edHost.Items.Add('1521');
  edHost.Items.Clear;
  for i := 0 to edServer.Items.Count - 1 do begin
    ServerInfo := TDirectServerInfo.Create;
    try
      ServerInfo.SetServerInfo(edServer.Items[i]);

      Host := ServerInfo.Host;
      Port := ServerInfo.Port;
      if ServerInfo.ServiceName <> '' then begin
        UseServiceName := True;
        Service := ServerInfo.ServiceName;
      end
      else begin
        UseServiceName := False;
        Service := ServerInfo.SID;
      end;
    finally
      ServerInfo.Free;
    end;

    if cbServiceName.Checked = UseServiceName then
      if edSID.Items.IndexOf(Service) < 0 then
        edSID.Items.Add(Service);
      if (Port <> '') and (edPort.Items.IndexOf(Port) < 0) then
        edPort.Items.Add(Port);
      if (Host <> '') and (edHost.Items.IndexOf(Host) < 0) then
        edHost.Items.Add(Host);
  end;
end;

{$IFDEF DBTOOLS}
class procedure TOraSessionEditorUtils.ExistingConnectionChange(const ConnectionName: string; const Connection: TOraSession);
var
  ConnectionStrList: TStrings;
  Str: string;
{$IFDEF NET}
  ServerInfo: TDirectServerInfo;
{$ENDIF}
begin
  ConnectionStrList := GetDBToolsService(TOraDesignUtils).GetConnectionStrList(ConnectionName);
  with Connection do begin
    Username := ConnectionStrList.Values['User Id'];
    Password := ConnectionStrList.Values['Password'];
    Options.UseUnicode := StrToBool(ConnectionStrList.Values['Unicode']);
    HomeName := ConnectionStrList.Values['Home'];
  {$IFDEF NET}
    Options.Direct := StrToBool(ConnectionStrList.Values['Direct']);
    if Options.Direct then begin
      ServerInfo := TDirectServerInfo.Create;
      try
        ServerInfo.Host := ConnectionStrList.Values['Server'];
        ServerInfo.Port := ConnectionStrList.Values['Port'];
        ServerInfo.ServiceName := ConnectionStrList.Values['ServiceName'];
        ServerInfo.SID := ConnectionStrList.Values['Sid'];

        Server := ServerInfo.GetServerInfo;
      finally
        ServerInfo.Free;
      end;
    end
    else
  {$ENDIF}
      Server := ConnectionStrList.Values['Server'];
    Str := ConnectionStrList.Values['Connect Mode'];
    if Str = 'SysOper' then
      ConnectMode := cmSysOper
    else
    if Str = 'SysDba' then
      ConnectMode := cmSysDBA
    else
      ConnectMode := cmNormal;
  end;
end;
{$ENDIF}

end.
